# This file is part of phishdetect-python:
# https://github.com/phishdetect/phishdetect-python
# See the file 'LICENSE' for copying permission.

class Model:

    def __init__(self, phishdetect):
        self._phishdetect = phishdetect



# This file is part of phishdetect-python:
# https://github.com/phishdetect/phishdetect-python
# See the file 'LICENSE' for copying permission.

from .model import Model
from ..endpoints import API_PATH

class Indicators(Model):

    def add(self, indicators, tags=[], enabled=True):
        """Add new indicators to PhishDetect Node.
        :param indicators: List of indicators, e.g. ["domain1.com", "domain2.com"].
        :param tags: List of tags to assign to all these indicators.
        """
        json = {
            "indicators": indicators,
            "tags": tags,
            "enabled": enabled,
        }
        return self._phishdetect.post(API_PATH["indicators_add"], json=json)

    def fetch(self):
        """Fetch the default set of indicators (should be, from last 6 months.)
        """
        return self._phishdetect.get(API_PATH["indicators_fetch"])

    def fetch_recent(self):
        """Fetch only the indicators from the last 24 hours.
        """
        return self._phishdetect.get(API_PATH["indicators_fetch_recent"])

    def fetch_all(self):
        """Fetch all the indicators stored in the Node.
        This API should be avoided unless strictly necessary.
        """
        return self._phishdetect.get(API_PATH["indicators_fetch_all"])        

    def details(self, sha256):
        """Retrieve details on a given indicator (by hash).
        :param sha256: SHA256 hash of the indicator.
        """
        return self._phishdetect.get(API_PATH["indicators_details"].format(sha256=sha256))

    def get_pending(self):
        """Fetch all the indicators that are marked as pending.
        """
        return self._phishdetect.get(API_PATH["indicators_pending"])

    def get_disabled(self):
        """Fetch all the indicators that are marked as disabled.
        """
        return self._phishdetect.get(API_PATH["indicators_disabled"])

    def toggle(self, indicators):
        """Send update of indicators to PhishDetect Node.
        :param indicators: List of hashed indicators in SHA256 format.
        """
        return self._phishdetect.post(API_PATH["indicators_toggle"], json=indicators)



# This file is part of phishdetect-python:
# https://github.com/phishdetect/phishdetect-python
# See the file 'LICENSE' for copying permission.

import requests
from urllib.parse import urljoin

from . import models
from .endpoints import API_PATH

class PhishDetect:

    def __init__(self, host, api_key=None):
        """Initialize a PhishDetect instance.

        :param host: Host to the PhishDetect Node.
        :param api_key: The API key to be used to authenticate requests.
        """
        self.host = host
        self.api_key = api_key
        self._session = requests.Session()
        self._session.headers = {"User-Agent": "phishdetect-python", "Connection": "close"}

        self.alerts = models.Alerts(self)
        self.indicators = models.Indicators(self)
        self.reports = models.Reports(self)
        self.users = models.Users(self)
        self.analyze = models.Analyze(self)

    def request(self, method, path, data=None, json=None, files=None, params=None):
        """Return the json from the resource requested at ``path``.
        :param method: HTTP method (mainly GET or POST).
        :param path: The path to the resource. This will be combined to the host
                     specified at creation.
        :param json: Any JSON encoded data to send with the request.
        :param data: Any data to send as body of the request.
        :param files: Files to upload.
        :param params: Query paramaters to send with the request.
        """
        url = urljoin(self.host, path)

        if not params:
            params = {}

        if self.api_key:
            params["key"] = self.api_key

        response = self._session.request(method=method, url=url, data=data,
            json=json, files=files, params=params)
        return response.json()

    def get(self, path, params=None):
        """Send a GET request and return the JSON response.
        :param path: Path to the resource to retrieve.
        """
        return self.request(method="GET", path=path, params=params)

    def post(self, path, data=None, json=None, files=None, params=None):
        """Send a POST request and return the JSON response.
        :param path: The path to the resource. This will be combined to the host
                     specified at creation.
        :param json: Any JSON encoded data to send with the request.
        :param data: Any data to send as body of the request.
        :param files: Files to upload.
        :param params: Query paramaters to send with the request.
        """
        return self.request(method="POST", path=path, data=data,
            json=json, files=files, params=params)

    def get_config(self):
        """Retrieve the config of specified the PhishDetect Node.
        """
        return self.get(API_PATH["config"])



# This file is part of phishdetect-python:
# https://github.com/phishdetect/phishdetect-python
# See the file 'LICENSE' for copying permission.

API_PATH = {
    "config":                  "/api/config/",
    "analyze_domain":          "/api/analyze/domain/",
    "analyze_url":             "/api/analyze/url/",
    "analyze_link":            "/api/analyze/link/",
    "analyze_html":            "/api/analyze/html/",
    "indicators_add":          "/api/indicators/add/",
    "indicators_fetch":        "/api/indicators/fetch/",
    "indicators_fetch_recent": "/api/indicators/fetch/recent/",
    "indicators_fetch_all":    "/api/indicators/fetch/all/",
    "indicators_details":      "/api/indicators/details/{sha256}/",
    "indicators_pending":      "/api/indicators/pending/",
    "indicators_disabled":     "/api/indicators/disabled/",
    "indicators_toggle":       "/api/indicators/toggle/",
    "alerts_add":              "/api/alerts/add/",
    "alerts_fetch":            "/api/alerts/fetch/",
    "reports_add":             "/api/reports/add/",
    "reports_fetch":           "/api/reports/fetch/",
    "reports_details":         "/api/reports/details/{uuid}/",
    "users_pending":           "/api/users/pending/",
    "users_active":            "/api/users/active/",
    "users_activate":          "/api/users/activate/{uuid}/",
    "users_deactivate":        "/api/users/deactivate/{uuid}/",
}



# This file is part of phishdetect-python:
# https://github.com/phishdetect/phishdetect-python
# See the file 'LICENSE' for copying permission.

from base64 import b64encode

from .model import Model
from ..endpoints import API_PATH

class Analyze(Model):

    def domain(self, domain):
        """Request the PhishDetect Node to statically analyze a domain name.
        :param domain: Domain to analyze.
        """
        return self._phishdetect.post(API_PATH["analyze_domain"], json={"url": domain})

    def url(self, url):
        """Request the PhishDetect Node to statically analyze a URL.
        :param url: URL to analyze.
        """
        return self._phishdetect.post(API_PATH["analyze_url"], json={"url": url})

    def link(self, url):
        """Request the PhishDetect Node to dynamically analyze a URL.
        :param url: URL to analyze.
        """
        return self._phishdetect.post(API_PATH["analyze_link"], json={"url": url})

    def html(self, url, html):
        """Request the PhishDetect Node to statically analyze an HTML page.
        :param url: URL of the HTML page.
        :param html: HTML of the page to analyze.
        """
        json = {
            "url": url,
            "html": b64encode(html.encode()).decode(),
        }
        return self._phishdetect.post(API_PATH["analyze_html"], json=json)



# This file is part of phishdetect-python:
# https://github.com/phishdetect/phishdetect-python
# See the file 'LICENSE' for copying permission.

from .alerts import Alerts
from .indicators import Indicators
from .reports import Reports
from .users import Users
from .analyze import Analyze



# This file is part of phishdetect-python:
# https://github.com/phishdetect/phishdetect-python
# See the file 'LICENSE' for copying permission.

from .model import Model
from ..endpoints import API_PATH

class Alerts(Model):

    def fetch(self, limit=0, offset=0):
        """Fetch all alerts stored by PhishDetect Node.
        :param limit: Set an integer to use as limit of records to retrieve.
        :param offset: Set an integer to use as offset of records to retrieve.
        """
        params = {
            "limit": limit,
            "offset": offset,
        }
        return self._phishdetect.get(API_PATH["alerts_fetch"], params=params)



# This file is part of phishdetect-python:
# https://github.com/phishdetect/phishdetect-python
# See the file 'LICENSE' for copying permission.

from .model import Model
from ..endpoints import API_PATH

class Users(Model):

    def get_pending(self):
        """Get list of users pending activation.
        """
        return self._phishdetect.get(API_PATH["users_pending"])

    def get_active(self):
        """Get list of active users.
        """
        return self._phishdetect.get(API_PATH["users_active"])

    def activate(self, uuid):
        """Activate pending user.
        :param uuid: UUID of the user to activate.
        """
        return self._phishdetect.get(API_PATH["users_activate"].format(uuid=uuid))

    def deactivate(self, uuid):
        """Deactivate existing user.
        :param uuid: UUID of the user to deactivate.
        """
        return self._phishdetect.get(API_PATH["users_deactivate"].format(uuid=uuid))



#!/usr/bin/env python3
# This file is part of phishdetect-python:
# https://github.com/phishdetect/phishdetect-python
# See the file 'LICENSE' for copying permission.

import os
from setuptools import setup, find_packages

__package_name__ = "phishdetect"
__version__ = "1.8.1"
__description__ = "This is a Python3 library and CLI tool " \
    "to easily interact with a PhishDetect Node API server"

requires = (
    "click",
    "requests",
)

def get_package_data(package):
    walk = [(dirpath.replace(package + os.sep, "", 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, "__init__.py"))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}

setup(
    name=__package_name__,
    version=__version__,
    author="Claudio Guarnieri",
    author_email="nex@nex.sx",
    description=__description__,
    long_description=__description__,
    install_requires=requires,
    packages=find_packages(),
    package_data=get_package_data("phishdetect"),
    scripts=["bin/phishdetect-cli",],
    include_package_data=True,
    keywords="security phishing phishdetect",
    license="MIT",
    classifiers=[
    ],
)
